@@ -110,11 +110,11 @@ class TestGaussianInference(unittest.TestCase):
         # alpha_k = weight_k * (c_alpha - K) + 1; c_alpha = sum(alpha0) + N
         c_alpha = 2.*20. + N; target_alpha = normalized_weights * (c_alpha - K) + 1
 
-        # beta_k = weight_k * (c_beta - K); c_beta = sum(beta0) + N
-        c_beta = 2.*30. + N; target_beta = normalized_weights * (c_beta - K)
+        # beta_k = beta_0 + N_k
+        target_beta = beta0 + normalized_weights * N
 
-        # nu_k = weight_k * (c_nu - K); c_nu = sum(nu0) + N
-        c_nu = 2.*40. + N; target_nu = normalized_weights * (c_nu - K)
+        # nu_k = nu_0 + N_k
+        target_nu = nu0 + normalized_weights * N
 
         target_m     = np.array([mean1, mean2])
         target_W     = np.array([np.linalg.inv(covariance1) / (target_nu[0] - 2.),
@@ -133,6 +133,23 @@ class TestGaussianInference(unittest.TestCase):
         np.testing.assert_almost_equal(re_covs             , np.array([covariance1, covariance2]))
         np.testing.assert_almost_equal(re_component_weights, normalized_weights                  )
 
+        # test default
+        self.assertRaisesRegexp(ValueError, 'Specify ``m``',
+                                GaussianInference, data, len(data) + 10)
+        # specify some m with too many components
+        GaussianInference(data, len(data) + 10, m=np.zeros((len(data) + 10, 2)))
+        GaussianInference(data, K)
+
+        # test 'first'
+        self.assertRaisesRegexp(ValueError, 'either.*components.*or.*initial_guess',
+                                GaussianInference, data, initial_guess='first')
+        vb = GaussianInference(data, K, initial_guess='first', alpha0=alpha0, beta0=beta0, nu0=nu0)
+        np.testing.assert_equal(vb.m, data[0:K])
+
+        # test 'random'
+        vb = GaussianInference(data, K, initial_guess='random', alpha0=alpha0, beta0=beta0, nu0=nu0)
+        self.assertFalse((vb.m == data[0:K]).any())
+
     def test_parameter_validation(self):
         target_mean = np.array((+1. , -4.))
         data = np.random.mtrand.multivariate_normal(target_mean, covariance1, size=500)
@@ -174,7 +191,7 @@ class TestGaussianInference(unittest.TestCase):
             print(cm.exception)
 
         # zero matrix not positive definite
-        with self.assertRaises(AssertionError) as cm:
+        with self.assertRaises(np.linalg.LinAlgError) as cm:
             GaussianInference(data, K, W=np.zeros((K, D, D)))
         print(cm.exception)
 
@@ -318,15 +335,14 @@ class TestGaussianInference(unittest.TestCase):
         sam = importance_sampling.ImportanceSampler(log_target, prop, rng = np.random.mtrand)
         sam.run(10**4)
 
-        weighted_samples = sam.history[:]
         # -----------------------------------------------------------------------------------------------
 
         rtol = .05
         atol = .01
         rtol_sigma = .12
 
-        weights = weighted_samples[:,0 ]
-        samples = weighted_samples[:,1:]
+        weights = sam.weights[:][:,0 ]
+        samples = sam.samples[:]
 
         clust = GaussianInference(samples, 2, weights=weights, m=np.vstack((prop_mean1,prop_mean2)))
         converged = clust.run(verbose=True)
@@ -442,27 +458,49 @@ def create_mixture(means, cov, ncomp):
     return MixtureDensity([Gauss(mu, cov) for mu in random_centers])
 
 class TestVBMerge(unittest.TestCase):
+    means = (np.array([1000.]), np.array([0.]))
+    cov = np.eye(1)
+    N = 500
+    # ten components around 0
+    input_mix = create_mixture(means[1:], cov, 10)
+    # one comp. around each mean
+    initial_guess = create_mixture(means, cov, 1)
+
     def setUp(self):
         np.random.seed(rng_seed)
 
-    def test_1d(self):
-        means = (np.array([1000.]), np.array([0.]))
-        cov = np.eye(1)
-        N = 500
-        # ten components around 0
-        input_mix = create_mixture(means[1:], cov, 10)
-        # one comp. around each mean
-        initial_guess = create_mixture(means, cov, 1)
-        # initial_guess.weights = np.ones(2) * 1e-5 / N
+    def test_initial_guess(self):
+        K = 3
+        means, _, _ = recover_gaussian_mixture(self.input_mix)
 
-        vb = VBMerge(input_mix, N=N, initial_guess=initial_guess)
+        # test default
+        self.assertRaisesRegexp(ValueError, 'Specify ``m``',
+                                VBMerge, self.input_mix, self.N, 32)
+        # specify some m with too many components
+        VBMerge(self.input_mix, self.N, 32, m=np.zeros((32, 1)))
+        VBMerge(self.input_mix, self.N, K)
+        # test 'first'
+        self.assertRaisesRegexp(ValueError, 'either.*components.*or.*initial_guess',
+                                VBMerge, self.input_mix, self.N, initial_guess='first')
+
+
+        vb = VBMerge(self.input_mix, self.N, K, initial_guess='first')
+        np.testing.assert_equal(vb.m, means[:K])
+
+        # test 'random'
+        vb = VBMerge(self.input_mix, self.N, K, initial_guess='random')
+
+        self.assertFalse((vb.m == means[:K]).any())
+
+    def test_1d(self):
+        vb = VBMerge(self.input_mix, N=self.N, initial_guess=self.initial_guess)
         vb.run(verbose=True)
         mix = vb.make_mixture()
         self.assertEqual(len(mix), 1)
         # means agree as m0 = 0 but correction from beta0
-        self.assertAlmostEqual(mix.components[0].mu[0], np.mean([c.mu for c in input_mix.components]), places=7)
+        self.assertAlmostEqual(mix.components[0].mu[0], np.mean([c.mu for c in self.input_mix.components]), places=7)
         # not identical due to large 'data' variance of the input means
-        self.assertAlmostEqual(mix.components[0].sigma[0], cov, delta=1.2)
+        self.assertAlmostEqual(mix.components[0].sigma[0], self.cov, delta=1.2)
 
     def test_bimodal(self):
         #Compress bimodal distribution with four components to two components.
@@ -651,7 +689,9 @@ class TestVBMerge(unittest.TestCase):
         weights = np.array([ 0.12644431,  0.87355569])
         components = [Gauss(m, c) for m,c in zip(means, cov)]
         input_components = MixtureDensity(components, weights)
-        vb = VBMerge(input_components, N=1e4,  components=2)
+        K = 2; dim = 2
+        old_initial_m = np.linspace(-1.,1., K*dim).reshape((K, dim))
+        vb = VBMerge(input_components, N=1e4,  components=2, m=np.linspace(-1.,1., K*dim).reshape((K, dim)))
         # compute (43) and (44) manually
         S = np.array([[ 0.01022336,  0.00301026],
                       [ 0.00301026,  0.00271089]])
@@ -673,19 +713,21 @@ class TestWishart(unittest.TestCase):
     nu = 8.3
 
     def test_Wishart_log_B(self):
-        W = np.eye(3)
+        D = 3
+        W = np.eye(D)
+        log_det = 0.0
         nu = 6
 
-        log_B = Wishart_log_B(W, nu)
+        log_B = Wishart_log_B(D, nu, log_det)
         self.assertAlmostEqual(log_B, np.log(0.00013192862453429398))
 
-        log_B = Wishart_log_B(self.W, self.nu)
+        log_B = Wishart_log_B(len(self.W), self.nu, np.log(np.linalg.det(self.W)))
         self.assertAlmostEqual(log_B, -19.6714760251454)
 
     def test_Wishart_H(self):
         # compare with Mathematica
 
-        self.assertAlmostEqual(Wishart_H(self.W, self.nu), 11.4262373965875)
+        self.assertAlmostEqual(Wishart_H(len(self.W), self.nu, np.log(np.linalg.det(self.W))), 11.4262373965875)
 
     def test_Wishart_expect_log_lambda(self):
-        self.assertAlmostEqual(Wishart_expect_log_lambda(self.W, self.nu), 6.24348627492751)
+        self.assertAlmostEqual(Wishart_expect_log_lambda(len(self.W), self.nu, np.log(np.linalg.det(self.W))), 6.24348627492751)
